import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;

import com.golden.gamedev.*;
import com.golden.gamedev.object.*;
import com.golden.gamedev.object.background.ColorBackground;

/**
 * Example of Pong Game.<br>
 * Introduce some collision groups,
 * such as CollisionGroupPrecise for more precise collision, and
 * CollisionBounds for collision with screen boundary.
 */
public class Sample4 extends Game {
	private PlayField 	playfield;
	private GameFont 	f;

	private SpriteGroup	BALL;	// ball group
	private SpriteGroup	PADDLE;	// paddle (player) group

	private Sprite		paddle;	// reference to player sprite
								// so we can move it based on keyboard input

	public void initResources() {
		f = fontManager.getFont(new Font("Verdana", Font.BOLD, 15));
		hideCursor();	// hide mouse pointer

		// creates playfield and all groups
		playfield = new PlayField(new ColorBackground(Color.BLACK));
		BALL = playfield.addGroup(new SpriteGroup("Ball"));
		PADDLE = playfield.addGroup(new SpriteGroup("Paddle"));


		// add two balls
		// first ball
		AnimatedSprite ball1 = new AnimatedSprite(getImages("ball.gif",4,3), 50, 200);
		ball1.setAnimate(true);
		ball1.setLoopAnim(true);
		ball1.getAnimationTimer().setDelay(getRandom(0, 4));
		ball1.setSpeed(getRandom(-3, 3), 4);

		// second ball
		AnimatedSprite ball2 = new AnimatedSprite(getImages("ball.gif",4,3), 200, 50);
		ball2.setAnimate(true);
		ball2.setLoopAnim(true);
		ball2.getAnimationTimer().setDelay(getRandom(0, 4));
		ball2.setSpeed(getRandom(-3, 3), 4);

		BALL.add(ball1);
		BALL.add(ball2);


		// the player
		paddle = new AnimatedSprite(getImages("paddle.gif", 2, 1), 300, 420);
		PADDLE.add(paddle);


		///////// add collision events into playfield /////////
		// collision between paddle and ball
		playfield.addCollisionGroup(PADDLE, BALL, new PaddleBallCollision());
		// collision between ball and ball
		playfield.addCollisionGroup(BALL, BALL, new BallBallCollision());
		// collision between ball and background
		playfield.addCollisionGroup(BALL, null,
			new BallBackgroundCollision(BALL, playfield.getBackground()));
	}

	public void update() {
		playfield.update();

		// detects keyboard input
		if (keyDown(KeyEvent.VK_LEFT)) {
			// left arrow, move paddle 6 pixels to left
			paddle.speedX = -6;

		} else if (keyDown(KeyEvent.VK_RIGHT)) {
			// right arrow, move paddle 6 pixels to right
			paddle.speedX = 6;

		} else {
			// no key pressed
			paddle.speedX = 0;
		}


		if (keyDown(KeyEvent.VK_ESCAPE)) {
			// quit game
			finish();
		}
	}

	public void render(Graphics2D g) {
		playfield.render(g);

		g.setColor(Color.YELLOW);
		f.drawString(g, "FPS = " + getCurrentFPS(), 20, 20);
	}

	/**
	 * Collision event between paddle and ball.
	 * Use precise collision. See CollisionGroupPrecise documentation.
	 */
	class PaddleBallCollision extends CollisionGroupPrecise {

		public void collided(Sprite s1, Sprite s2) {
			// revert position
			revertPosition1();
			// animate paddle
			((AnimatedSprite) s1).setAnimate(true);

			// bounce ball based on hit position with paddle
			s2.setSpeed(((s2.getX() + (s2.getWidth()/2)) -
						(s1.getX() + (s1.getWidth()/2))) / 5,
						-4);
			// max horizontal speed = 5
			if (s2.getSpeedX() > 5) s2.setSpeedX(5);
			if (s2.getSpeedX() < -5) s2.setSpeedX(-5);

			// set ball position on top of paddle
			s2.setY(s1.getY() - s2.getHeight());
	   }

	}
	/**
	 * Collision event between ball and ball.
	 */
	class BallBallCollision extends CollisionGroupPrecise {

		public void collided(Sprite s1, Sprite s2) {
			// revert position
			revertPosition1();
			revertPosition2();

			// reverse speed
			if (getCollisionSide() == LEFT_RIGHT_COLLISION ||
				getCollisionSide() == RIGHT_LEFT_COLLISION) {
				s1.speedX = -s1.speedX;
				s2.speedX = -s2.speedX;

			} else {
				s1.speedY = -s1.speedY;
				s2.speedY = -s2.speedY;
			}
		}

	}

	/**
	 * Collision event between balls and background.
	 */
	class BallBackgroundCollision extends CollisionBounds {

		public BallBackgroundCollision(SpriteGroup group, Background backgr) {
			super(group, backgr);
		}

		public void collided(Sprite s) {
			// revert ball position
			revertPosition1();

			if (isCollideSide(BOTTOM_COLLISION) ||
				isCollideSide(TOP_COLLISION)) {
				// top down collision
				s.speedY *= -1;
			}

			if (isCollideSide(LEFT_COLLISION) ||
				isCollideSide(RIGHT_COLLISION)) {
				// left right collision
				s.speedX *= -1;
			}
		}
	}

	public static void main(String[] args) {
		Game game = new Sample4();
		game.setup(new Dimension(640, 480), true);
		game.start();
	}

}